@@ -11,6 +11,8 @@ from PetThermoTools.MELTS import *
 
 Names = {'liquid1': '_Liq',
         'liquid2': '_Liq2',
+        'liquid3': '_Liq3',
+        'liquid4': '_Liq4',
         'olivine1': '_Ol',
         'olivine2': '_Ol2',
         'clinopyroxene1': '_Cpx',
@@ -33,6 +35,9 @@ Names = {'liquid1': '_Liq',
         'apatite2': '_Apa2'}
 
 Names_MM = {'liq1': '_Liq',
+            'liq2': '_Liq2',
+            'liq3': '_Liq3',
+            'liq4': '_Liq4',
             'ol1': '_Ol',
             'ol2': '_Ol2',
             'cpx1': '_Cpx',
@@ -43,8 +48,44 @@ Names_MM = {'liq1': '_Liq',
             'g2': '_Grt2',
             'fsp1': '_Plag',
             'fsp2': '_Plag2',
-            'spn1': '_Sp',
-            'spn2': '_Sp2'}
+            'spl1': '_Sp',
+            'spl2': '_Sp2'}
+
+def to_float(x):
+    if x is None:
+        return None
+    if isinstance(x, (int, float)):
+        return float(x)
+    if isinstance(x, (list, tuple)):
+        return [float(v) for v in x]
+    if isinstance(x, np.ndarray):
+        return x.astype(float)
+    return x  # leave unchanged if unexpected type
+
+def label_results(Result,label):
+    Results = Result.copy()
+    new_out = {}
+    if  label == "CO2":
+        for r in Results:
+            new_out['CO2 = ' + str(Results[r]['Input']['comp']['CO2_Liq']) + ' wt%'] = Results[r].copy()
+        new_out = dict(sorted(new_out.items(), key=lambda x: float(x[0].split('=')[1].split(' ')[1])))
+    elif label == "pressure" or label == "P" or label == "P_bar":
+        for r in Results:
+            new_out['P = ' + str(Results[r]['Input']['P_bar']) + ' bars'] = Results[r].copy()
+        new_out = dict(sorted(new_out.items(), key=lambda x: float(x[0].split('=')[1].split(' ')[1])))
+    elif label == "fO2":
+        for r in Results:
+            new_out['fO2 = ' + Results[r]['Input']['fO2_buffer'] + ' ' + str(round(Results[r]['Input']['fO2_offset'],2))] = Results[r].copy()
+        new_out = dict(sorted(new_out.items(), key=lambda x: float(x[0].split('=')[1].split(' ')[2])))
+    elif label == 'H2O':
+        for r in Results:
+            new_out['H2O = ' + str(Results[r]['Input']['comp']['H2O_Liq']) + ' wt%'] = Results[r].copy()
+        new_out = dict(sorted(new_out.items(), key=lambda x: float(x[0].split('=')[1].split(' ')[1])))
+    
+    if len(new_out) == 0:
+        new_out = Results.copy()
+    
+    return new_out
 
 def supCalc(Model = "MELTSv1.0.2", bulk = None, phase = None, T_C = None, P_bar = None,
              Fe3Fet_Liq = None, H2O_Liq = None, CO2_Liq = None, fO2_buffer = None, fO2_offset = None, 
